<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2008
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects. 
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:redirect="Http://xml.apache.org/xalan/redirect"
 extension-element-prefixes="redirect">

  <xsl:output method="xml" indent="yes" encoding="UTF-8"/>

  <xsl:include href="xradar://stylesheets/radar/statics/process/master-merge.xsl"/>

  <xsl:param	name="log"/>
  <xsl:param	name="input-report-file"/>
  <xsl:variable name="emma-report-doc" select="document($input-report-file)"/>

  <xsl:variable name="report-source" select="'EMMA'"/>
  <xsl:variable name="report-source-link" select="'http://emma.sourceforge.net/'"/>



  <xsl:template
	 match="Stats">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
      <xsl:if test="not(test-metrics)">
        <xsl:choose>
          <xsl:when test="$log = 'true'">
            <xsl:message>
    test metrics: <xsl:value-of select="$input-report-file"/> 'stats' element exist <xsl:value-of select="name($emma-report-doc/report/stats)"/>
            </xsl:message>
          </xsl:when>
        </xsl:choose>
        <test-metrics>
          <xsl:apply-templates
			select="$emma-report-doc//package" mode="test-stats">
            <xsl:with-param
							name="package-name">
              <xsl:value-of select="../@name"/>
            </xsl:with-param>
            <xsl:with-param
							name="source-statements">
              <xsl:value-of select="ncss"/>
            </xsl:with-param>
            <xsl:with-param
							name="log">
              <xsl:value-of select="$log"/>
            </xsl:with-param>
          </xsl:apply-templates>
          <xsl:choose>
            <xsl:when test="$log = 'true'">
              <xsl:message>
    test metrics package: <xsl:value-of select="../@name"/> source-statements:<xsl:value-of select="ncss"/>
          </xsl:message>
          </xsl:when>
          </xsl:choose>
        </test-metrics>
      </xsl:if>
    </xsl:copy>
  </xsl:template>

  <xsl:template
	match="test-metrics">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
      <xsl:apply-templates
			select="$emma-report-doc//package" mode="test-stats">
        <xsl:with-param
				name="package-name">
          <xsl:value-of select="../../@name"/>
        </xsl:with-param>
        <xsl:with-param
			name="source-statements">
          <xsl:value-of select="../ncss"/>
        </xsl:with-param>
		<xsl:with-param
			name="log">
          <xsl:value-of select="$log"/>
        </xsl:with-param>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>


  <xsl:template
	 match="package" mode="test-stats">
    <xsl:param name="package-name"/>
    <xsl:param name="source-statements"/>
    <xsl:choose>
      <xsl:when test="$log = 'true'">
        <xsl:message>
          test-stats package <xsl:value-of select="$package-name" /> source-statements:<xsl:value-of select="$source-statements" />
        </xsl:message>
      </xsl:when>
    </xsl:choose>
    <xsl:if test="@name = $package-name">
      <xsl:variable name="line-coverage" select="$emma-report-doc//package[@name = $package-name]/coverage[@type='line, %']/@value"/>
      <!-- use the computed result in percentage "value="76%  (194.8/256)" -->
      <xsl:variable name="line-rate" select="number(substring-before($line-coverage, '%')) div 100"/>
      <!-- use the first value from bracket "value="76%  (194.8/256)" -->
      <xsl:variable name="statement-count" select="substring-before(substring-after($line-coverage, '('), '/')"/>
      <line-coverage>
        <xsl:value-of select="$line-rate"/>
      </line-coverage>
      <source-statements-covered>
        <xsl:value-of select="$statement-count"/>
      </source-statements-covered>
      
      <xsl:choose>
        <xsl:when test="$log = 'true'">
          <xsl:message>
          test-stats package <xsl:value-of select="$package-name" /> coverage:<xsl:value-of select="$line-rate" /> tested-lines: <xsl:value-of select="$statement-count" />
      </xsl:message>
        </xsl:when>
      </xsl:choose>

      <xsl:variable name="method-value" select="$emma-report-doc//package[@name = $package-name]/coverage[@type='method, %']/@value"/>
      <!-- use the computed result in percentage "value="76%  (194.8/256)" -->
      <xsl:variable name="method-rate" select="number(substring-before($method-value, '%')) div 100"/>
      <!-- use the first value from bracket "value="76%  (194.8/256)" -->
      <xsl:variable name="method-count" select="substring-before(substring-after($method-value, '('), '/')"/>
      <method-coverage>
        <xsl:value-of select="$method-rate"/>
      </method-coverage>
      <methods-referenced>
        <xsl:value-of select="$method-count"/>
      </methods-referenced>

    </xsl:if>
  </xsl:template>


  <!-- line coverage for  package 
			<package name="org.xradar.test.a">
				<coverage type="class, %" value="100% (1/1)"/>
				<coverage type="method, %" value="67%  (4/6)"/>
				<coverage type="block, %" value="76%  (103/136)"/>
				<coverage type="line, %" value="77%  (33/43)"/>
				...
			</package>
			

  -->
			  <xsl:template name="package-addition"/>


  <!-- coverage for class
		   <package name="org.xradar.test.a">
			<coverage type="class, %" value="100% (1/1)"/>
			<coverage type="method, %" value="67%  (4/6)"/>
			<coverage type="block, %" value="76%  (103/136)"/>
			<coverage type="line, %" value="77%  (33/43)"/>

			<srcfile name="A1.java">
			  <coverage type="class, %" value="100% (1/1)"/>
			  <coverage type="method, %" value="67%  (4/6)"/>
			  <coverage type="block, %" value="76%  (103/136)"/>
			  <coverage type="line, %" value="77%  (33/43)"/>

			  <class name="A1">
				<coverage type="class, %" value="100% (1/1)"/>
				<coverage type="method, %" value="67%  (4/6)"/>
				<coverage type="block, %" value="76%  (103/136)"/>
				<coverage type="line, %" value="77%  (33/43)"/>
-->

  <xsl:template
	 match="Class">
    <xsl:variable name="class-name" select="normalize-space(node())"/>
    <xsl:variable name="package-name" select="../../@name"/>
    <xsl:variable name="class-name-without-package" select="substring-before(@sourceFile,'.java')"/>
    <!--   
    <xsl:variable name="package-name" select="substring-before($class-name, concat('.',$class-name-without-package))"/>
	-->
    <xsl:variable name="current-emma-class" select="$emma-report-doc//package[@name=$package-name]//class[@name=$class-name-without-package]"/>

    <xsl:copy>
      <xsl:apply-templates select="@*|node()">
        <xsl:with-param name="current-emma-class" select="$current-emma-class"/>
		<xsl:with-param name="log" select="$log"/>
      </xsl:apply-templates>

      <xsl:if test="$current-emma-class">
        <xsl:if test="../../Stats/test-metrics">
          <xsl:variable name="line-coverage" select="$current-emma-class/coverage[@type='line, %']/@value"/>
          <!--  select="$emma-report-doc//class[@name = $class-name-without-package]/coverage[@type='line, %']/@value"/> -->
          <xsl:variable name="line-rate" select="number(substring-before($line-coverage, '%')) div 100"/>
          <xsl:variable name="statement-count" select="substring-before(substring-after($line-coverage, '('), '/')"/>
          
          <xsl:variable name="method-value" select="$current-emma-class/coverage[@type='method, %']/@value"/>
          <!-- use the computed result in percentage "value="76%  (194.8/256)" -->
          <xsl:variable name="method-rate" select="number(substring-before($method-value, '%')) div 100"/>
          <!-- use the first value from bracket "value="76%  (194.8/256)" -->
          <xsl:variable name="method-count" select="substring-before(substring-after($method-value, '('), '/')"/>
          <test-metrics>
            <line-coverage>
              <xsl:value-of select="$line-rate"/>
            </line-coverage>
            <source-statements-covered>
              <xsl:value-of select="$statement-count"/>
            </source-statements-covered>
            <method-coverage>
              <xsl:value-of select="$method-rate"/>
            </method-coverage>
            <methods-referenced>
              <xsl:value-of select="$method-count"/>
            </methods-referenced>
          </test-metrics>
          <xsl:choose>
            <xsl:when test="$log = 'true'">
			<xsl:message>
              class: <xsl:value-of select="$class-name" /> coverage:<xsl:value-of select="$line-rate" /> tested-lines: <xsl:value-of select="$statement-count" />
			</xsl:message>
			</xsl:when>
          </xsl:choose>
        </xsl:if>
      </xsl:if>
    </xsl:copy>
  </xsl:template>

  <xsl:template
	match="functions">
    <xsl:param name="current-emma-class"/>
    <xsl:copy>
      <xsl:apply-templates select="@*|node()">
        <xsl:with-param name="current-emma-class" select="$current-emma-class"/>
		<xsl:with-param name="log" select="$log"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>

  <!-- coverage for function
		   <package name="org.xradar.test.a">
			<coverage type="class, %" value="100% (1/1)"/>
			<coverage type="method, %" value="67%  (4/6)"/>
			<coverage type="block, %" value="76%  (103/136)"/>
			<coverage type="line, %" value="77%  (33/43)"/>

			<srcfile name="A1.java">
			  <coverage type="class, %" value="100% (1/1)"/>
			  <coverage type="method, %" value="67%  (4/6)"/>
			  <coverage type="block, %" value="76%  (103/136)"/>
			  <coverage type="line, %" value="77%  (33/43)"/>

			  <class name="A1">
				<coverage type="class, %" value="100% (1/1)"/>
				<coverage type="method, %" value="67%  (4/6)"/>
				<coverage type="block, %" value="76%  (103/136)"/>
				<coverage type="line, %" value="77%  (33/43)"/>
				            <method name="main (String []): void">
					  <coverage type="method, %" value="0%   (0/1)"/>
					  <coverage type="block, %" value="0%   (0/28)"/>
					  <coverage type="line, %" value="0%   (0/8)"/>
					</method>
					<method name="methodWithParameters2 (Boolean, String, Vector, int): String">
					  <coverage type="method, %" value="0%   (0/1)"/>
					  <coverage type="block, %" value="0%   (0/2)"/>
					  <coverage type="line, %" value="0%   (0/1)"/>
					</method>
-->
  <xsl:template
	match="function">
    <xsl:param name="current-emma-class"/>
    <xsl:variable name="class-name" select="normalize-space(../../node())"/>
    <xsl:variable name="class-name-without-package" select="substring-before(../../@sourceFile,'.java')"/>
    <xsl:variable name="method-name-with-params" select="normalize-space(substring-after(name,concat($class-name,'.')))"/>
    <xsl:variable name="method-name" select="substring-before($method-name-with-params,'(')"/>

    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
      <xsl:if
		test="($current-emma-class//method[contains(@name,$method-name)]) or ($method-name = $class-name-without-package)">
        <test-metrics>
        <xsl:for-each
      select="$current-emma-class//method[starts-with(translate(@name,' ',''),$method-name-with-params)]">
            <!-- select="$emma-report-doc//class[@name=$class-name-without-package]/method">  -->
        <xsl:variable name="orig-param-set" select="substring-before(substring-after($method-name-with-params,'('),')')"/>
        <xsl:variable name="emma-param-set" select="translate(substring-before(substring-after(@name,'('),')'),' ','')"/>
        <xsl:if test="$orig-param-set = $emma-param-set">
				<xsl:variable name="value" select="coverage[@type='line, %']/@value"/>
				<xsl:variable name="line-rate" select="number(substring-before($value, '%')) div 100"/>
				<xsl:variable name="statement-count" select="substring-before(substring-after($value, '('), '/')"/>
              
        <xsl:variable name="method-value" select="coverage[@type='method, %']/@value"/>
        <!-- use the computed result in percentage "value="76%  (194.8/256)" -->
        <xsl:variable name="method-rate" select="number(substring-before($method-value, '%')) div 100"/>
        <!-- use the first value from bracket "value="76%  (194.8/256)" -->
        <xsl:variable name="method-count" select="substring-before(substring-after($method-value, '('), '/')"/>

        <line-coverage>
					<xsl:value-of select="$line-rate"/>
				</line-coverage>
				<source-statements-covered>
					<xsl:value-of select="$statement-count"/>
				</source-statements-covered>
        <method-coverage>
          <xsl:value-of select="$method-rate"/>
        </method-coverage>
        <methods-referenced>
          <xsl:value-of select="$method-count"/>
        </methods-referenced>
		<xsl:choose>
		    <xsl:when test="$log = 'true'">
		      <xsl:message>
                 test-stats function <xsl:value-of select="$method-name" /> coverage:<xsl:value-of select="$line-rate" /> tested-lines: <xsl:value-of select="$statement-count" />
		      </xsl:message>
		    </xsl:when>
		</xsl:choose>
            </xsl:if>
          </xsl:for-each>
        </test-metrics>
      </xsl:if>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
