<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:redirect="http://xml.apache.org/xalan/redirect"
    extension-element-prefixes="redirect">

    <xsl:output method="xml" indent="yes" encoding="UTF-8"/>

    <xsl:param name="radar-config-file"/>
    <xsl:param name="target-directory"/>
    <xsl:param name="log"/>


    <xsl:variable name="radar-config-doc" select="document($radar-config-file)"/>


    <xsl:template match="radar-report">

    	<xsl:choose>
            <xsl:when test="$log = 'true'">
                <xsl:message>
                    Radar-config:        <xsl:value-of        select="$radar-config-file"/>
                    target-directory:    <xsl:value-of        select="$target-directory"/>
                    log:<xsl:value-of select="$log"/>
                </xsl:message>
            </xsl:when>
        </xsl:choose>
        <xsl:variable name="packages-element" select="packages"/>
        <xsl:variable name="languages-element" select="languages"/>
        <xsl:apply-templates select="$radar-config-doc/radar-config/defined-metrics/metric">
            <xsl:with-param name="packages-element" select="$packages-element"/>
             <!-- David Stokar -->
            <xsl:with-param name="languages-element" select="$languages-element"/>
             <!-- David Stokar -->
        </xsl:apply-templates>

        <xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		    <xsl:variable name="subsystem-name" select="normalize-space(name)"/>

            <redirect:write file="{$target-directory}/subsystem_quality_report_{$subsystem-name}.xml">
                <xsl:apply-templates select="$radar-config-doc/radar-config/defined-metrics/metric">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                </xsl:apply-templates>
            </redirect:write>
        </xsl:for-each>

		<xsl:for-each select="$packages-element/package">
		    <xsl:variable name="package-name" select="@name"/>
		    <xsl:variable name="subsystem-name" select="@subsystem"/>
		    <redirect:write file="{$target-directory}/package_quality_report_{$package-name}.xml">
                <xsl:apply-templates select="$radar-config-doc/radar-config/defined-metrics/metric">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                    <xsl:with-param name="package-name" select="$package-name"/>
                </xsl:apply-templates>
            </redirect:write>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="metric">
        <xsl:param name="packages-element"/>
         <!-- David Stokar -->
        <xsl:param name="languages-element"/>
         <!-- David Stokar -->
        <xsl:param name="subsystem-name" select="''"/>
        <xsl:param name="package-name" select="''"/>

        <xsl:copy>
            <xsl:apply-templates select="@*"/>

            <xsl:if test="@type='aggregate'">
                <value><xsl:call-template name="aggregate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="value"/>
                    <xsl:with-param name="metric-element" select="current()"/>
                </xsl:call-template></value>
            </xsl:if>

            <xsl:if test="@type='value'">
                <value><xsl:call-template name="calculate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="value"/>
                </xsl:call-template></value>
            </xsl:if>

            <xsl:apply-templates select="metric">
                <xsl:with-param name="packages-element" select="$packages-element"/>
                 <!-- David Stokar -->
                <xsl:with-param name="languages-element" select="$languages-element"/>
                 <!-- David Stokar -->
                <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
				<xsl:with-param name="package-name" select="$package-name"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>

    <xsl:template name="aggregate-value">
        <xsl:param name="packages-element"/>
         <!-- David Stokar -->
        <xsl:param name="languages-element"/>
         <!-- David Stokar -->
        <xsl:param name="subsystem-name" select="''"/>
        <xsl:param name="package-name" select="''"/>
        <xsl:param name="value-element"/>
        <xsl:param name="metric-element"/>
        <xsl:param name="node-number" select="1"/>
        <xsl:param name="value" select="0"/>

        <xsl:variable name="abbreviation" select="$value-element/metric[position() = $node-number]/@abbreviation"/>
        <xsl:variable name="significance" select="$value-element/metric[position() = $node-number]/@significance"/>


        <xsl:choose>
            <xsl:when test="$node-number > count($value-element/metric)">
                <xsl:value-of select="$value"/>
            </xsl:when>

            <xsl:otherwise>
                <xsl:call-template name="aggregate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="$value-element"/>
                    <xsl:with-param name="metric-element" select="$metric-element"/>
                    <xsl:with-param name="node-number" select="$node-number + 1"/>
                    <xsl:with-param name="value">
                        <xsl:call-template name="perform-arithmic-operation">
                            <xsl:with-param name="value-1" select="$value"/>
                            <xsl:with-param name="operator" select="'add'"/>
                            <xsl:with-param name="value-2">
                                <xsl:call-template name="perform-arithmic-operation">
                                    <xsl:with-param name="value-1" select="$significance"/>
                                    <xsl:with-param name="operator" select="'multiply'"/>
                                    <xsl:with-param name="value-2">
                                        <xsl:choose>
                                            <xsl:when test="$metric-element/metric[@abbreviation = $abbreviation]/@type = 'value'">
                                                <xsl:call-template name="calculate-value">
                                                    <xsl:with-param name="packages-element" select="$packages-element"/>
                                                    <!-- David Stokar -->
													<xsl:with-param name="languages-element" select="$languages-element"/>
													<!-- END OF David Stokar -->
                                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
													<xsl:with-param name="package-name" select="$package-name"/>
													
                                                    <xsl:with-param name="value-element" select="$metric-element/metric[@abbreviation = $abbreviation]/value"/>
                                                </xsl:call-template>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:call-template name="aggregate-value">
                                                    <xsl:with-param name="packages-element" select="$packages-element"/>
                                                    <!-- David Stokar -->
                                                    <xsl:with-param name="languages-element" select="$languages-element"/>
                                                    <!-- END OF David Stokar -->
                                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
													<xsl:with-param name="package-name" select="$package-name"/>
                                                    <xsl:with-param name="value-element" select="$metric-element/metric[@abbreviation = $abbreviation]/value"/>
                                                    <xsl:with-param name="metric-element" select="$metric-element/metric[@abbreviation = $abbreviation]"/>
                                                </xsl:call-template>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:with-param>
                                </xsl:call-template>
                            </xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="calculate-value">
        <xsl:param name="packages-element"/>
        <!-- David Stokar -->
        <xsl:param name="languages-element"/>
        <!-- END OF David Stokar -->
        <xsl:param name="subsystem-name" select="''"/>
        <xsl:param name="package-name" select="''"/>
        <xsl:param name="value-element"/>
        <xsl:param name="node-number" select="1"/>
        <xsl:param name="value" select="0"/>
        <xsl:param name="operator" select="''"/>

        <xsl:choose>
            <xsl:when test="$node-number > count($value-element/*)">
                <xsl:value-of select="$value"/>
            </xsl:when>

			<!-- The operator element -->
            <xsl:when test="count($value-element/*[(position()= $node-number) and (name()='operator')]) > 0">
                <xsl:variable name="operator-name" select="$value-element/*[position()= $node-number]/@value"/>
                <xsl:call-template name="calculate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                    <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <!-- END OF David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                    <xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="$value-element"/>
                    <xsl:with-param name="node-number" select="$node-number + 1"/>
                    <xsl:with-param name="value" select="$value"/>
                    <xsl:with-param name="operator" select="$operator-name"/>
                </xsl:call-template>
            </xsl:when>

			<!-- The number element -->
            <xsl:when test="count($value-element/*[position()= $node-number and name()='number']) >0">
                <xsl:variable name="number-value" select="$value-element/*[position()= $node-number]/@value"/>
                <xsl:call-template name="calculate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                    <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <!-- END OF David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="$value-element"/>
                    <xsl:with-param name="node-number" select="$node-number + 1"/>
                    <xsl:with-param name="value" select="$number-value"/>
                    <xsl:with-param name="operator" select="''"/>
                </xsl:call-template>
            </xsl:when>

			<!-- The stat element -->
            <xsl:when test="count($value-element/*[position()= $node-number and name()='stat']) >0">
                <xsl:variable name="group-element" select="$value-element/*[position()= $node-number]/@group"/>
                <xsl:variable name="stat-element" select="$value-element/*[position()= $node-number]/@value"/>

				<xsl:variable name="new-value">
					<xsl:choose>
						<xsl:when test="$package-name != ''">
							<xsl:value-of select="sum($packages-element/package[@name = $package-name]/statistics/*[name() = $group-element]/*[(name() = $stat-element) and (node() != 'NaN')])"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/*[name() = $group-element]/*[(name() = $stat-element) and (node() != 'NaN')])"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

                <xsl:call-template name="calculate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                    <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <!-- END OF David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="$value-element"/>
                    <xsl:with-param name="node-number" select="$node-number + 1"/>
                    <xsl:with-param name="value">
                        <xsl:call-template name="perform-arithmic-operation">
                            <xsl:with-param name="value-1" select="$value"/>
                            <xsl:with-param name="operator" select="$operator"/>
                            <xsl:with-param name="value-2" select="$new-value"/>
                        </xsl:call-template>
                    </xsl:with-param>
                    <xsl:with-param name="operator" select="''"/>
                </xsl:call-template>
            </xsl:when>

			<!-- The count element -->
            <xsl:when test="count($value-element/*[position()= $node-number and name()='count']) >0">
                <xsl:variable name="group-element" select="$value-element/*[position()= $node-number]/@group"/>
                <xsl:variable name="stat-element" select="$value-element/*[position()= $node-number]/@value"/>
                <xsl:variable name="condition-value" select="$value-element/*[position()= $node-number]/@condition-value"/>
                <xsl:variable name="condition-not-value" select="$value-element/*[position()= $node-number]/@condition-not-value"/>

				<xsl:variable name="new-value">
					<xsl:choose>
						<xsl:when test="$package-name != ''">
							<xsl:choose>
                        		<xsl:when test="$value-element/*[position()= $node-number]/@condition-value">
									<xsl:value-of select="count($packages-element/package[@name = $package-name]/statistics/*[name() = $group-element]/*[(name() = $stat-element) and (normalize-space(.) = $condition-value)])"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="count($packages-element/package[@name = $package-name]/statistics/*[name() = $group-element]/*[(name() = $stat-element) and (normalize-space(.) != $condition-not-value)])"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise>
							<xsl:choose>
                        		<xsl:when test="$value-element/*[position()= $node-number]/@condition-value">
									<xsl:value-of select="count($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/*[name() = $group-element]/*[(name() = $stat-element) and (normalize-space(.) = $condition-value)])"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="count($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/*[name() = $group-element]/*[(name() = $stat-element) and (normalize-space(.) != $condition-not-value)])"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

				<xsl:call-template name="calculate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                    <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <!-- END OF David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="$value-element"/>
                    <xsl:with-param name="node-number" select="$node-number + 1"/>
                    <xsl:with-param name="value">
                        <xsl:call-template name="perform-arithmic-operation">
                            <xsl:with-param name="value-1" select="$value"/>
                            <xsl:with-param name="operator" select="$operator"/>
                            <xsl:with-param name="value-2" select="$new-value"/>
                        </xsl:call-template>
                    </xsl:with-param>
                    <xsl:with-param name="operator" select="''"/>
                </xsl:call-template>
            </xsl:when>

			<!-- The count-classes element -->
			<xsl:when test="count($value-element/*[position()= $node-number and name()='count-classes']) >0">
                <xsl:variable name="group-element" select="$value-element/*[position()= $node-number]/@group"/>
                <xsl:variable name="stat-element" select="$value-element/*[position()= $node-number]/@value"/>
                <xsl:variable name="max-value" select="$value-element/*[position()= $node-number]/@max-value"/>
				<xsl:variable name="new-value">
					<xsl:choose>
						<xsl:when test="$package-name != ''">
							<xsl:value-of select="count($packages-element/package[@name = $package-name]/classes/child::class/child::statistics/*[name() = $group-element]/*[(name() = $stat-element) and (normalize-space(.) &lt; $max-value)])"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="count($packages-element/package[starts-with(@subsystem,$subsystem-name)]/classes/child::class/child::statistics/*[name() = $group-element]/*[(name() = $stat-element) and (normalize-space(.) &lt; $max-value)])"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
                <xsl:call-template name="calculate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                    <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <!-- END OF David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="$value-element"/>
                    <xsl:with-param name="node-number" select="$node-number + 1"/>
                    <xsl:with-param name="value">
                        <xsl:call-template name="perform-arithmic-operation">
                            <xsl:with-param name="value-1" select="$value"/>
                            <xsl:with-param name="operator" select="$operator"/>
                            <xsl:with-param name="value-2" select="$new-value"/>
                        </xsl:call-template>
                    </xsl:with-param>
                    <xsl:with-param name="operator" select="''"/>
                </xsl:call-template>
            </xsl:when>

			<!-- The packages element -->
			<xsl:when test="count($value-element/*[position()= $node-number and name()='packages']) >0">

				<xsl:variable name="new-value">
					<xsl:choose>
						<xsl:when test="$package-name != ''">
							<xsl:value-of select="count($packages-element/package[@name = $package-name])"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="count($packages-element/package[starts-with(@subsystem,$subsystem-name)])"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>

                <xsl:call-template name="calculate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                    <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <!-- END OF David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="$value-element"/>
                    <xsl:with-param name="node-number" select="$node-number + 1"/>
                    <xsl:with-param name="value">
                        <xsl:call-template name="perform-arithmic-operation">
                            <xsl:with-param name="value-1" select="$value"/>
                            <xsl:with-param name="operator" select="$operator"/>
                            <xsl:with-param name="value-2" select="$new-value"/>
                        </xsl:call-template>
                    </xsl:with-param>
                    <xsl:with-param name="operator" select="''"/>
                </xsl:call-template>
            </xsl:when>

			<!-- The value element -->
            <xsl:when test="count($value-element/*[position()= $node-number and name()='value']) >0">
                <xsl:call-template name="calculate-value">
                    <xsl:with-param name="packages-element" select="$packages-element"/>
                    <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <!-- END OF David Stokar -->
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="package-name" select="$package-name"/>
                    <xsl:with-param name="value-element" select="$value-element"/>
                    <xsl:with-param name="node-number" select="$node-number + 1"/>
                    <xsl:with-param name="value">
                        <xsl:call-template name="perform-arithmic-operation">
                            <xsl:with-param name="value-1" select="$value"/>
                            <xsl:with-param name="operator" select="$operator"/>
                            <xsl:with-param name="value-2">
                                <xsl:call-template name="calculate-value">
                                    <xsl:with-param name="packages-element" select="$packages-element"/>
                                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
									<xsl:with-param name="package-name" select="$package-name"/>
                                    <xsl:with-param name="value-element" select="$value-element/*[position()= $node-number]"/>
                                </xsl:call-template>
                            </xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                    <xsl:with-param name="operator" select="''"/>
                </xsl:call-template>
            </xsl:when>


			<!-- David Stokar -->
 			<!-- The lang element -->
             <xsl:when test="count($value-element/*[position()= $node-number and name()='lang']) >0">
                 <xsl:variable name="language-name" select="$value-element/*[position()= $node-number]/@name"/>
                 <xsl:variable name="stat-element" select="$value-element/*[position()= $node-number]/@value"/>
                <xsl:variable name="new-value" select="number($languages-element/language[@name=$language-name]/@code)" />
 
                 <xsl:call-template name="calculate-value">
                     <xsl:with-param name="packages-element" select="$packages-element"/>
                     <!-- David Stokar -->
                    <xsl:with-param name="languages-element" select="$languages-element"/>
                    <!-- END OF David Stokar -->
                     <xsl:with-param name="languages-element" select="$languages-element"/>
                     <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
 		            <xsl:with-param name="package-name" select="$package-name"/>
                     <xsl:with-param name="value-element" select="$value-element"/>
                     <xsl:with-param name="node-number" select="$node-number + 1"/>
                     <xsl:with-param name="value">
                         <xsl:call-template name="perform-arithmic-operation">
                             <xsl:with-param name="value-1" select="$value"/>
                             <xsl:with-param name="operator" select="$operator"/>
                             <xsl:with-param name="value-2" select="$new-value"/>
                         </xsl:call-template>
                     </xsl:with-param>
                     <xsl:with-param name="operator" select="''"/>
                 </xsl:call-template>
             </xsl:when>
             <!-- End David Stokar -->

			<!-- Return value from recorursion-->
            <xsl:otherwise>
                <xsl:value-of select="$value"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <xsl:template name="perform-arithmic-operation">
        <xsl:param name="value-1"/>
        <xsl:param name="operator"/>
        <xsl:param name="value-2"/>

        <xsl:choose>
            <xsl:when test="$operator = 'add'">
                <xsl:value-of select="$value-1 + $value-2"/>
            </xsl:when>
            <xsl:when test="$operator = 'subtract'">
                <xsl:value-of select="$value-1 - $value-2"/>
            </xsl:when>
            <xsl:when test="$operator = 'divide'">
                <xsl:choose>
                    <xsl:when test="$value-2 = 0">1</xsl:when>
                    <xsl:otherwise><xsl:value-of select="$value-1 div $value-2"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$operator = 'multiply'">
                <xsl:value-of select="$value-1 * $value-2"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$value-1 + $value-2"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- Copy all nodes from here. Can be overridden by a more specific XPath -->
    <xsl:template match="@*|node()">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()"/>
        </xsl:copy>
    </xsl:template>


</xsl:stylesheet>
